
--local inspect = require "inspect"

-- TODO LG: przeniesc do liba
local bind = function(func, ...)
	local args = {...}
	return function()
		return func(table.unpack(args))
	end
end

-- TODO LG: przeniesc do liba i ustawic jako domyslna funkcje
function HID.isGroupActive(groupName)
	return HID.activeGroups[groupName]
end

local function IsAnyShiftDown() return HID.isButtonDown(HID.KB_LSHIFT) or HID.isButtonDown(HID.KB_RSHIFT) end
local function IsAnyControlDown() return HID.isButtonDown(HID.KB_LCONTROL) or HID.isButtonDown(HID.KB_RCONTROL) end
local function IsAnyAltDown() return HID.isButtonDown(HID.KB_LALT) or HID.isButtonDown(HID.KB_RALT) end
local function IsAnyModDown() return IsAnyShiftDown() or IsAnyControlDown() or IsAnyAltDown() end

local gameConfigPadId = 1
if HardwarePlatform.getName() == "Win" then
	gameConfigPadId = 0
end

--[[setmetatable(HID,
{
	index = function(table, key)
		print("Lua: missing '"..key.."' key in HID")
	end
})]]

local function ToFuncBool(val)
	if type(val) == "function" then
		return val
	elseif not val then
		return function () return false	end
	else
		return function () return val end
	end
end

local function ToFuncFloat(val)
	if type(val) == "function" then
		return val
	elseif not val then
		return function () return 0.0	end
	else
		return function () return val end
	end
end

local function ButtonView(view)

--	print(inspect(view))
	local wasPressed = ToFuncBool(view.wasPressed)
	local isDown = ToFuncBool(view.isDown)
	local wasReleased = ToFuncBool(view.wasReleased)

	assert(type(wasPressed) == "function")
	assert(type(isDown) == "function")
	assert(type(wasReleased) == "function")

	assert(type(wasPressed()) == "boolean")
	assert(type(isDown()) == "boolean")
	assert(type(wasReleased()) == "boolean")

	return function() return wasPressed(), isDown(), wasReleased() end
end

local function AnalogView(view)
	if view.x or view.y then
		local x = ToFuncBool(view.x)
		local y = ToFuncBool(view.y)

		assert(type(x) == "function")
		assert(type(y) == "function")

		assert(type(x()) == "number")
		assert(type(y()) == "number")

		return function() return x(), y() end
	else
		local right = ToFuncBool(view.right)
		local left = ToFuncBool(view.left)
		local up = ToFuncBool(view.up)
		local down = ToFuncBool(view.down)

		assert(type(right) == "function")
		assert(type(left) == "function")
		assert(type(up) == "function")
		assert(type(down) == "function")

		assert(type(right()) == "boolean")
		assert(type(left()) == "boolean")
		assert(type(up()) == "boolean")
		assert(type(down()) == "boolean")

		return function() return right(), left(), up(), down() end
	end
end

---------------------------------------------------------------------------------------------------------------------

local dpadAndArrowsPressedAsAnalog = AnalogView
	{
		right = bind(HID.wasButtonPressed, HID.PB_RIGHT, HID.KB_RIGHT),
		left = bind(HID.wasButtonPressed, HID.PB_LEFT, HID.KB_LEFT),
		up = bind(HID.wasButtonPressed, HID.PB_UP, HID.KB_UP),
		down = bind(HID.wasButtonPressed, HID.PB_DOWN, HID.KB_DOWN)
	}
local dpadAndArrowsDownAsAnalog = AnalogView
	{
		right = bind(HID.isButtonDown, HID.PB_RIGHT, HID.KB_RIGHT),
		left = bind(HID.isButtonDown, HID.PB_LEFT, HID.KB_LEFT),
		up = bind(HID.isButtonDown, HID.PB_UP, HID.KB_UP),
		down = bind(HID.isButtonDown, HID.PB_DOWN, HID.KB_DOWN)
	}
-------------------------------------------------------------------------------------------------------------------

local Win =
{
	info =
	{
		isActive = function(self) return HardwarePlatform.getName() == "Win" end
	},
}

local actions =
{
--	Win = Win,
	play =
	{
		info =
		{
			isActive = function() return not HID.isGroupActive("gameConfig") end
		},

		IA_BERSERK_HOLD		= bind(HID.isButtonDown, HID.PB_DOWN, HID.KB_DOWN),
		IA_SPEEDER_HOLD		= bind(HID.isButtonDown, HID.PB_RIGHT, HID.KB_RIGHT),
		IA_PARACHUTE_HOLD	= bind(HID.isButtonDown, HID.PB_A, HID.KB_Z, HID.KB_SPACE),
		IA_PARACHUTE_UP_HOLD= bind(HID.isButtonDown, HID.PB_UP, HID.KB_UP),
		IA_OLD_HOLD			= bind(HID.wasButtonPressed, HID.PB_LEFT, HID.KB_LEFT),

		IA_JUMP_PRESSED		= bind(HID.wasButtonPressed, HID.PB_A, HID.KB_SPACE, HID.KB_Z),
		IA_JUMP_RELEASED	= bind(HID.wasButtonReleased, HID.PB_A, HID.KB_SPACE, HID.KB_Z),
	},


	menu =
	{
		info =
		{
			isActive = function() return not HID.isGroupActive("gameConfig") end
		},

		IA_MENU_UP		= bind(HID.wasButtonPressed, HID.PB_UP, HID.KB_UP),
		IA_MENU_DOWN	= bind(HID.wasButtonPressed, HID.PB_DOWN, HID.KB_DOWN),
		IA_MENU_RIGHT	= bind(HID.wasButtonPressed, HID.PB_RIGHT, HID.KB_RIGHT),
		IA_MENU_LEFT	= bind(HID.wasButtonPressed, HID.PB_LEFT, HID.KB_LEFT),

		IA_PAUSE = bind(HID.wasButtonPressed, HID.PB_START, HID.KB_TAB, HID.KB_ESCAPE, HID.KB_ENTER),

		IA_MENU_ACCEPT = function()	return false,	HID.isButtonDown(HID.PB_A, HID.KB_Z), HID.wasButtonReleased(HID.PB_A, HID.KB_Z) end,
		IA_MENU_BACK   = function()	return false,	HID.isButtonDown(HID.PB_START, HID.PB_B, HID.KB_X),	HID.wasButtonReleased(HID.PB_START, HID.PB_B, HID.KB_X)	end,
		IA_CUTSCENE_NEXT = bind(HID.wasButtonPressed, HID.PB_A, HID.KB_Z),
		IA_MINIMAP_CONTINUE = bind(HID.wasButtonReleased, HID.PB_START, HID.KB_TAB, HID.KB_ESCAPE), --todo wywalic uzycie w kodzie i podpiac pod gui
	},

	map =
	{
		info =
		{
			isActive = function() return not HID.isGroupActive("gameConfig") end
		},

		IA_MAP_SELECT_LEVEL	= bind(HID.wasButtonPressed, HID.PB_A, HID.KB_Z),

		IA_MAP_GNOME_MOVE = dpadAndArrowsDownAsAnalog,
		IA_MAP_GNOME_MOVE_START = dpadAndArrowsPressedAsAnalog,

		IA_MAP_EXIT_GAME = bind(HID.wasButtonReleased, HID.KB_ESCAPE), --android only
		IA_MAP_CHANGE_CAM_Z = bind(HID.isButtonDown, HID.MB_RBUTTON),

		IA_CLOSE_ABOUT = bind(HID.wasButtonPressed, HID.KB_ESCAPE), --todo wywalic uzycie w kodzie i podpiac pod gui
		IA_CLOSE_CREDITS = bind(HID.wasButtonPressed, HID.KB_ESCAPE), --todo wywalic uzycie w kodzie i podpiac pod gui

		IA_CLOSE_FB = bind(HID.wasButtonPressed, HID.KB_ESCAPE), --todo wywalic uzycie w kodzie i podpiac pod gui
		IA_CONNECT_FB = bind(HID.wasButtonPressed, HID.KB_ENTER), --todo wywalic uzycie w kodzie i podpiac pod gui

		IA_GOAL_CLOSE = bind(HID.wasButtonPressed, HID.KB_ESCAPE), --todo wywalic uzycie w kodzie i podpiac pod gui
		IA_GOAL_PLAY = bind(HID.wasButtonPressed, HID.KB_ENTER), --todo wywalic uzycie w kodzie i podpiac pod gui

		IA_GPG_CLOSE = bind(HID.wasButtonPressed, HID.KB_ESCAPE), --todo wywalic uzycie w kodzie i podpiac pod gui
		IA_GPG_PLAY = bind(HID.wasButtonPressed, HID.KB_ENTER), --todo wywalic uzycie w kodzie i podpiac pod gui

		IA_LOCK_CLOSE = bind(HID.wasButtonPressed, HID.KB_ESCAPE), --todo wywalic uzycie w kodzie i podpiac pod gui

		IA_OPTIONS_CLOSE = bind(HID.wasButtonPressed, HID.KB_ESCAPE), --todo wywalic uzycie w kodzie i podpiac pod gui
		IA_SHOP_CLOSE = bind(HID.wasButtonPressed, HID.KB_ESCAPE), --todo wywalic uzycie w kodzie i podpiac pod gui
		IA_SKIP_CLOSE = bind(HID.wasButtonPressed, HID.KB_ESCAPE), --todo wywalic uzycie w kodzie i podpiac pod gui
		IA_SKIP_CONFIRM_CLOSE = bind(HID.wasButtonPressed, HID.KB_ESCAPE), --todo wywalic uzycie w kodzie i podpiac pod gui

		IA_START_MENU_EXIT = bind(HID.wasButtonReleased, HID.KB_ESCAPE), --android only
	},

	debug =
	{
		info =
		{
			isActive = function() return not HID.isGroupActive("gameConfig") end
		},

		gsp =
		{
			IA_GSP_ENTER		= bind(HID.wasPadButtonPressed, 1, HID.PB_START, HID.KB_B),
			IA_GSP_EXIT			= bind(HID.wasPadButtonPressed, 1, HID.PB_START, HID.KB_B),
			IA_GSP_ADD			= bind(HID.wasPadButtonPressed, 1, HID.PB_X, HID.KB_G),
			IA_GSP_REMOVE		= bind(HID.wasPadButtonPressed, 1, HID.PB_Y, HID.KB_H),

			IA_GSP_LEFT			= bind(HID.wasPadButtonPressed, 1, HID.PB_LEFT, HID.KB_OPEN_BRACE),
			IA_GSP_RIGHT		= bind(HID.wasPadButtonPressed, 1, HID.PB_RIGHT, HID.KB_CLOSE_BRACE),
			IA_GSP_PROFILE	= bind(HID.wasPadButtonPressed, 1, HID.PB_RTRIGGER, HID.KB_O),
			IA_GSP_SAVE			= bind(HID.wasPadButtonPressed, 1, HID.PB_UP, HID.KB_I),
			IA_GSP_SET_POS = bind(HID.wasButtonPressed, HID.MB_LBUTTON),
			IA_GSP_SPEED_UP		= bind(HID.isButtonDown, HID.KB_LEFT_SHIFT, HID.KB_RIGHT_SHIFT),
		},

		noModifiers =
		{
			info = { isActive = function() return not IsAnyModDown() end },

			IA_DEBUG_VIEW_SPEED_UP = bind(HID.isButtonDown, HID.PB_A, HID.KB_Z),
			IA_DEBUG_VIEW_SET_VIRTUAL_CHECKPOINT   = bind(HID.wasButtonPressed, HID.MB_LBUTTON),
			IA_DEBUG_VIEW_UNSET_VIRTUAL_CHECKPOINT = bind(HID.wasButtonPressed, HID.MB_RBUTTON),

			IA_DEBUG_OPAQUE_MODE = bind(HID.wasButtonPressed, HID.KB_O),
			IA_DEBUG_FAKE_MOVES = bind(HID.wasButtonPressed, HID.KB_F),
			IA_DEBUG_CHUNK_SWITCHING = function() return false, HID.isButtonDown(HID.KB_J), HID.wasButtonReleased(HID.KB_J) end,
			IA_DEBUG_NEXT_CHUNK = bind(HID.wasButtonPressed, HID.KB_RIGHT),
			IA_DEBUG_PREV_CHUNK = bind(HID.wasButtonPressed, HID.KB_LEFT),
			IA_DEBUG_NEXT_CHECKPOINT = function() return HID.wasButtonPressed(HID.KB_N) end,
			IA_DEBUG_USE_DYNAMIC_OBJ_CHUNKS = bind(HID.wasButtonPressed, HID.KB_H),
			IA_DEBUG_VIEW = bind(HID.wasButtonPressed, HID.PB_X, HID.KB_A),
		},

		withAlt =
		{
			info = { isActive = function() return IsAnyAltDown() end },
		},

		withShift =
		{
			info ={ isActive = function() return IsAnyShiftDown() end },
			IA_DEBUG_NOISE_EFFECT = bind(HID.wasButtonPressed, HID.KB_N),
		},

		withControl =
		{
			info = { isActive = function() return IsAnyControlDown() end },
		},

		withControlAlt =
		{
			info = { isActive = function() return IsAnyControlDown() and IsAnyAltDown() end },
		},

		withControlShift =
		{
			info = { isActive = function() return IsAnyControlDown() and IsAnyShiftDown() end },
		},

		withAllModifiers =
		{
			info = { isActive = function() return IsAnyControlDown() and IsAnyShiftDown() and IsAnyAltDown() end },
		},

		IA_DEBUG_VIEW_MOVE = dpadAndArrowsDownAsAnalog,

		IA_DEBUG_TRIANGULATE = bind(HID.wasButtonPressed, HID.KB_T),
		IA_DEBUG_TRIANGULATE_MODIFIER = IsAnyShiftDown,
		IA_DEBUG_OFF_CAMERA_Z_OFFSET = bind(HID.wasButtonPressed, HID.KB_C),
		IA_DEBUG_OFF_CAMERA_Z_OFFSET_MODIFIER = IsAnyShiftDown,

		IA_DEBUG_PROFILE_UPDATE = bind(HID.wasButtonPressed, HID.PB_SELECT),

		IA_DEBUG_START_AGAIN = function() return IsAnyAltDown() and HID.wasButtonPressed(HID.KB_R) end,
		IA_DEBUG_START_AGAIN_NO_RELOAD_MODIFIER = IsAnyControlDown,
		IA_DEBUG_LOAD_DEBUG_COLORS = function() return IsAnyAltDown() and HID.wasButtonPressed(HID.KB_D) end,
		IA_DEBUG_LOAD_DEBUG_COLORS_NO_RELOAD_MODIFIER = IsAnyControlDown,

		IA_DEBUG_NEW_BATCHES = bind(HID.wasPadButtonPressed, 1, HID.PB_Y, HID.KB_F1),

		IA_DEBUG_DRAW_PHYSICS = bind(HID.wasButtonPressed, HID.PB_Y, HID.KB_S),
		IA_DEBUG_PROFILE = bind(HID.wasPadButtonPressed, 1, HID.PB_RBUMPER, HID.KB_L),

		IA_DEBUG_LAYER0 = bind(HID.wasButtonPressed, HID.KB_0),
		IA_DEBUG_LAYER1 = bind(HID.wasButtonPressed, HID.KB_1),
		IA_DEBUG_LAYER2 = bind(HID.wasButtonPressed, HID.KB_2),
		IA_DEBUG_LAYER3 = bind(HID.wasButtonPressed, HID.KB_3),
		IA_DEBUG_LAYER4 = bind(HID.wasButtonPressed, HID.KB_4),
		IA_DEBUG_LAYER5 = bind(HID.wasButtonPressed, HID.KB_5),
		IA_DEBUG_LAYER6 = bind(HID.wasButtonPressed, HID.KB_6),
		IA_DEBUG_LAYER7 = bind(HID.wasButtonPressed, HID.KB_7),
		IA_DEBUG_LAYER8 = bind(HID.wasButtonPressed, HID.KB_8),
		IA_DEBUG_LAYER9 = bind(HID.wasButtonPressed, HID.KB_9),
		IA_DEBUG_LAYER_MODIFIER = IsAnyControlDown,

		IA_DEBUG_LOCK_LEVELS_MODIFIER = IsAnyControlDown,
		IA_DEBUG_UNLOCK_LEVELS_MODIFIER = IsAnyShiftDown,
		IA_DEBUG_TOGGLE_LEVEL_SKIP_MODIFIER = IsAnyAltDown,

		IA_DEBUG_INTRO_RELOAD = bind(HID.wasButtonPressed, HID.KB_SPACE),
		IA_DEBUG_INPUT_ACTIONS_RELOAD = bind(HID.wasButtonPressed, HID.KB_F10),
	},

	IA_GAME_CONFIG_RELOAD = bind(HID.wasButtonPressed, HID.KB_F9),
	IA_GAME_CONFIG_ENTER = bind(HID.wasPadButtonPressed, gameConfigPadId, HID.PB_LTRIGGER, HID.KB_1),

	gameConfig =
	{
		info = {isActive = bind(HID.isGroupActive, "gameConfig") },

		IA_GAME_CONFIG_SPEED_UP = bind(HID.isButtonDown, gameConfigPadId, HID.PB_RTRIGGER, HID.KB_W),

		IA_GAME_CONFIG_ACCEPT = bind(HID.wasPadButtonPressed, gameConfigPadId, HID.PB_A, HID.KB_Z),
		IA_GAME_CONFIG_SET_DEFAULT = bind(HID.wasPadButtonPressed, gameConfigPadId, HID.PB_B, HID.KB_X),
		IA_GAME_CONFIG_INC = bind(HID.wasPadButtonPressed, gameConfigPadId, HID.PB_RBUMPER, HID.KB_S),
		IA_GAME_CONFIG_DEC = bind(HID.wasPadButtonPressed, gameConfigPadId, HID.PB_LBUMPER, HID.KB_A),
		IA_GAME_CONFIG_CURSOR_UP = function() return HID.wasPadButtonPressed(gameConfigPadId, HID.PB_UP, HID.KB_UP), HID.isPadButtonDown(gameConfigPadId, HID.PB_UP, HID.KB_UP), false end,
		IA_GAME_CONFIG_CURSOR_DOWN = function() return HID.wasPadButtonPressed(gameConfigPadId, HID.PB_DOWN, HID.KB_DOWN), HID.isPadButtonDown(gameConfigPadId, HID.PB_DOWN, HID.KB_DOWN), false end,

		IA_GAME_CONFIG_CURSOR_RIGHT = bind(HID.wasPadButtonPressed, gameConfigPadId, HID.PB_RIGHT, HID.KB_RIGHT),
		IA_GAME_CONFIG_CURSOR_LEFT = bind(HID.wasPadButtonPressed, gameConfigPadId, HID.PB_LEFT, HID.KB_LEFT),

	},

	unicornPrototype =
	{
		IA_DASH = bind(HID.wasButtonPressed, HID.KB_DOWN, HID.PB_DOWN),
	},
}
return actions
